/* KcomRegistryRepository.cc */
/* Created by Enomoto Sanshiro on 27 May 2000. */
/* Last updated by Enomoto Sanshiro on 27 May 2000. */


#include <iostream>
#include <string>
#include "KcomRegistry.hh"
#include "KcomRegistryRepository.hh"

using namespace std;


TKcomRegistryRepository::TKcomRegistryRepository(void)
{
}

TKcomRegistryRepository::~TKcomRegistryRepository()
{
}

TKcomRegistry* TKcomRegistryRepository::Create(void)
{
    return new TKcomRegistry(new TKcomRegistryEntry(""));
}

void TKcomRegistryRepository::Save(TKcomRegistry* Registry, ostream& OutputStream)
{
    WriteChildEntries(Registry->GetEntry("/"), OutputStream);
}

TKcomRegistry* TKcomRegistryRepository::Load(istream& InputStream) throw(TKcomException)
{
    TParaCxxTokenTable TokenTable;
    TParaTokenizer Tokenizer(InputStream, &TokenTable);

    TKcomRegistryEntry* RootEntry = new TKcomRegistryEntry("");
    TKcomRegistry* Registry = new TKcomRegistry(RootEntry);

    try {
	ReadChildEntries(RootEntry, Tokenizer);
    }
    catch (TScriptException &e) {
	delete Registry;
	throw TKcomException(
	    "TKcomRegistryRepository::Load()",
	    "script exception: " + e.Message()
	);
    }

    return Registry;
}

void TKcomRegistryRepository::WriteChildEntries(TKcomRegistryEntry* Entry, ostream& os, string Indent)
{
    const vector<TKcomRegistryEntry*>& ChildEntryList = Entry->ChildEntryList();
    for (unsigned i = 0; i < ChildEntryList.size(); i++) {
	TKcomRegistryEntry* ChildEntry = ChildEntryList[i];

	os << Indent << "[" << ChildEntry->Name();
	if (! ChildEntry->Value().empty()) {
	    os << " = \"" << ChildEntry->Value() << "\"";
	}
	os << "]";
	
	if (! ChildEntry->ChildEntryList().empty()) {
	    os << " {" << endl;
	    WriteChildEntries(ChildEntry, os, Indent + "    ");
	    os << Indent << "}";
	    
	    if (Indent.empty()) {
		os << endl;
	    }
	}

	os << endl;
    }    
}

void TKcomRegistryRepository::ReadChildEntries(TKcomRegistryEntry* Entry, TParaTokenizer& Tokenizer) throw(TScriptException)
{
    TParaToken Token;
    Tokenizer.Next().MustBe("[");
    
    (Token = Tokenizer.Next()).MustBe(TParaToken::ttIdentifier);
    string Name = Token.AsString();
    
    string Value;
    if (Tokenizer.LookAhead().IsNot("]")) {
	Tokenizer.Next().MustBe("=");
	Value = Tokenizer.Next().RemoveQuotation('"').AsString();
    }
    
    Tokenizer.Next().MustBe("]");
    
    TKcomRegistryEntry* ChildEntry = new TKcomRegistryEntry(Name, Value);
    Entry->AddChildEntry(ChildEntry);

    if (Tokenizer.LookAhead().Is("{")) {
	Tokenizer.Next().MustBe("{");
	ReadChildEntries(ChildEntry, Tokenizer);
	Tokenizer.Next().MustBe("}");
    }

    if (Tokenizer.LookAhead().Is("[")) {
	ReadChildEntries(Entry, Tokenizer);
    }
}
